@@ -6,7 +6,7 @@
 # Copyright (C) 2006-2009, 2011-2014, 2017 Lukáš Lalinský
 # Copyright (C) 2008 Gary van der Merwe
 # Copyright (C) 2008 amckinle
-# Copyright (C) 2008-2010, 2014-2015, 2018-2020 Philipp Wolfer
+# Copyright (C) 2008-2010, 2014-2015, 2018-2021 Philipp Wolfer
 # Copyright (C) 2009 Carlin Mangar
 # Copyright (C) 2010 Andrew Barnert
 # Copyright (C) 2011-2014 Michael Wiencek
@@ -16,7 +16,7 @@
 # Copyright (C) 2013 Ionuț Ciocîrlan
 # Copyright (C) 2013 brainz34
 # Copyright (C) 2013-2014, 2017 Sophist-UK
-# Copyright (C) 2013-2015, 2017-2019 Laurent Monin
+# Copyright (C) 2013-2015, 2017-2021 Laurent Monin
 # Copyright (C) 2016 Rahul Raturi
 # Copyright (C) 2016 Simon Legner
 # Copyright (C) 2016 Suhas
@@ -25,7 +25,8 @@
 # Copyright (C) 2018 Bob Swift
 # Copyright (C) 2018 virusMac
 # Copyright (C) 2019 Joel Lintunen
-# Copyright (C) 2020 Gabriel Ferreira
+# Copyright (C) 2020 Julius Michaelis
+# Copyright (C) 2020-2021 Gabriel Ferreira
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -43,9 +44,9 @@
 
 
 import argparse
-from collections import defaultdict
+from concurrent.futures import ThreadPoolExecutor
 from functools import partial
-from itertools import chain
+import itertools
 import logging
 import os.path
 import platform
@@ -67,9 +68,13 @@ from picard import (
     PICARD_FANCY_VERSION_STR,
     PICARD_ORG_NAME,
     acoustid,
-    config,
     log,
 )
+from picard.acousticbrainz import (
+    ABExtractor,
+    ab_extractor_callback,
+    ab_feature_extraction,
+)
 from picard.acoustid.manager import AcoustIDManager
 from picard.album import (
     Album,
@@ -84,6 +89,10 @@ from picard.cluster import (
     UnclusteredFiles,
 )
 from picard.collection import load_user_collections
+from picard.config import (
+    get_config,
+    setup_config,
+)
 from picard.config_upgrade import upgrade_config
 from picard.const import (
     USER_DIR,
@@ -92,7 +101,6 @@ from picard.const import (
 from picard.const.sys import (
     IS_FROZEN,
     IS_HAIKU,
-    IS_MACOS,
     IS_WIN,
 )
 from picard.dataobj import DataObject
@@ -111,10 +119,11 @@ from picard.util import (
     decode_filename,
     encode_filename,
     is_hidden,
+    iter_files_from_objects,
     mbid_validate,
+    normpath,
     process_events_iter,
     thread,
-    uniqify,
     versions,
     webbrowser2,
 )
@@ -165,7 +174,8 @@ class Tagger(QtWidgets.QApplication):
 
         super().__init__(sys.argv)
         self.__class__.__instance = self
-        config._setup(self, picard_args.config_file)
+        setup_config(self, picard_args.config_file)
+        config = get_config()
         theme.setup(self)
 
         self._cmdline_files = picard_args.FILE
@@ -178,21 +188,18 @@ class Tagger(QtWidgets.QApplication):
         if picard_args.debug or "PICARD_DEBUG" in os.environ:
             self.set_log_level(logging.DEBUG)
 
-        # FIXME: Figure out what's wrong with QThreadPool.globalInstance().
-        # It's a valid reference, but its start() method doesn't work.
-        self.thread_pool = QtCore.QThreadPool(self)
+        # Default thread pool
+        self.thread_pool = ThreadPoolExecutor()
 
         # Provide a separate thread pool for operations that should not be
         # delayed by longer background processing tasks, e.g. because the user
         # expects instant feedback instead of waiting for a long list of
         # operations to finish.
-        self.priority_thread_pool = QtCore.QThreadPool(self)
-        self.priority_thread_pool.setMaxThreadCount(1)
+        self.priority_thread_pool = ThreadPoolExecutor(max_workers=1)
 
         # Use a separate thread pool for file saving, with a thread count of 1,
         # to avoid race conditions in File._save_and_rename.
-        self.save_thread_pool = QtCore.QThreadPool(self)
-        self.save_thread_pool.setMaxThreadCount(1)
+        self.save_thread_pool = ThreadPoolExecutor(max_workers=1)
 
         if not IS_WIN:
             # Set up signal handling
@@ -217,16 +224,12 @@ class Tagger(QtWidgets.QApplication):
             signal.signal(signal.SIGINT, self.signal)
             signal.signal(signal.SIGTERM, self.signal)
 
-        if IS_MACOS:
-            # On macOS it is not common that the global menu shows icons
-            self.setAttribute(QtCore.Qt.AA_DontShowIconsInMenus)
-
         # Setup logging
         log.debug("Starting Picard from %r", os.path.abspath(__file__))
         log.debug("Platform: %s %s %s", platform.platform(),
                   platform.python_implementation(), platform.python_version())
         log.debug("Versions: %s", versions.as_string())
-        log.debug("Configuration file path: %r", config.config.fileName())
+        log.debug("Configuration file path: %r", config.fileName())
 
         log.debug("User directory: %r", os.path.abspath(USER_DIR))
 
@@ -241,7 +244,7 @@ class Tagger(QtWidgets.QApplication):
         # translated
         setup_gettext(localedir, config.setting["ui_language"], log.debug)
 
-        upgrade_config(config.config)
+        upgrade_config(config)
 
         self.webservice = WebService()
         self.mb_api = MBAPIHelper(self.webservice)
@@ -254,6 +257,11 @@ class Tagger(QtWidgets.QApplication):
         self._acoustid.init()
         self.acoustidmanager = AcoustIDManager(acoustid_api)
 
+        # Setup AcousticBrainz extraction
+        self.ab_extractor = ABExtractor()
+
+        self.enable_menu_icons(config.setting['show_menu_icons'])
+
         # Load plugins
         self.pluginmanager = PluginManager()
         if not self._no_plugins:
@@ -268,6 +276,7 @@ class Tagger(QtWidgets.QApplication):
             self.pluginmanager.load_plugins_from_directory(USER_PLUGIN_DIR)
 
         self.browser_integration = BrowserIntegration()
+        self.browser_integration.listen_port_changed.connect(self.listen_port_changed)
 
         self._pending_files_count = 0
         self.files = {}
@@ -285,6 +294,9 @@ class Tagger(QtWidgets.QApplication):
         if self.autoupdate_enabled:
             self.updatecheckmanager = UpdateCheckManager(parent=self.window)
 
+    def enable_menu_icons(self, enabled):
+        self.setAttribute(QtCore.Qt.AA_DontShowIconsInMenus, not enabled)
+
     def register_cleanup(self, func):
         self.exit_cleanup.append(func)
 
@@ -313,20 +325,20 @@ class Tagger(QtWidgets.QApplication):
                 authorization_code, scopes,
                 partial(self.on_mb_authorization_finished, callback))
         else:
-            callback(False)
+            callback(False, None)
 
-    def on_mb_authorization_finished(self, callback, successful=False):
+    def on_mb_authorization_finished(self, callback, successful=False, error_msg=None):
         if successful:
             self.webservice.oauth_manager.fetch_username(
                 partial(self.on_mb_login_finished, callback))
         else:
-            callback(False)
+            callback(False, error_msg)
 
     @classmethod
-    def on_mb_login_finished(self, callback, successful):
+    def on_mb_login_finished(self, callback, successful, error_msg):
         if successful:
             load_user_collections()
-        callback(successful)
+        callback(successful, error_msg)
 
     def mb_logout(self):
         self.webservice.oauth_manager.revoke_tokens()
@@ -336,11 +348,7 @@ class Tagger(QtWidgets.QApplication):
         """Move `files` to tracks on album `albumid`."""
         if album is None:
             album = self.load_album(albumid)
-        if album.loaded:
-            album.match_files(files)
-        else:
-            for file in list(files):
-                file.move(album.unmatched_files)
+        album.match_files(files)
 
     def move_file_to_album(self, file, albumid):
         """Move `file` to a track on album `albumid`."""
@@ -349,9 +357,8 @@ class Tagger(QtWidgets.QApplication):
     def move_file_to_track(self, file, albumid, recordingid):
         """Move `file` to recording `recordingid` on album `albumid`."""
         album = self.load_album(albumid)
-        album.run_when_loaded(partial(file.move, album.unmatched_files), always=True)
-        album.run_when_loaded(partial(album.match_files, [file],
-                                      recordingid=recordingid))
+        file.match_recordingid = recordingid
+        album.match_files([file])
 
     def create_nats(self):
         if self.nats is None:
@@ -375,9 +382,9 @@ class Tagger(QtWidgets.QApplication):
         self.stopping = True
         log.debug("Picard stopping")
         self._acoustid.done()
-        self.thread_pool.waitForDone()
-        self.save_thread_pool.waitForDone()
-        self.priority_thread_pool.waitForDone()
+        self.thread_pool.shutdown()
+        self.save_thread_pool.shutdown()
+        self.priority_thread_pool.shutdown()
         self.browser_integration.stop()
         self.webservice.stop()
         self.run_cleanup()
@@ -390,14 +397,20 @@ class Tagger(QtWidgets.QApplication):
             del self._cmdline_files
 
     def run(self):
-        if config.setting["browser_integration"]:
-            self.browser_integration.start()
+        self.update_browser_integration()
         self.window.show()
         QtCore.QTimer.singleShot(0, self._run_init)
         res = self.exec_()
         self.exit()
         return res
 
+    def update_browser_integration(self):
+        config = get_config()
+        if config.setting["browser_integration"]:
+            self.browser_integration.start()
+        else:
+            self.browser_integration.stop()
+
     def event(self, event):
         if isinstance(event, thread.ProxyToMainEvent):
             event.run()
@@ -412,7 +425,8 @@ class Tagger(QtWidgets.QApplication):
             return 1
         return super().event(event)
 
-    def _file_loaded(self, file, target=None, remove_file=False):
+    def _file_loaded(self, file, target=None, remove_file=False, unmatched_files=None):
+        config = get_config()
         self._pending_files_count -= 1
         if self._pending_files_count == 0:
             self.window.set_sorting(True)
@@ -428,54 +442,68 @@ class Tagger(QtWidgets.QApplication):
             self.unclustered_files.add_file(file)
             return
 
+        file_moved = False
         if not config.setting["ignore_file_mbids"]:
-            recordingid = file.metadata['musicbrainz_recordingid']
+            recordingid = file.metadata.getall('musicbrainz_recordingid')
+            recordingid = recordingid[0] if recordingid else ''
             is_valid_recordingid = mbid_validate(recordingid)
 
-            albumid = file.metadata['musicbrainz_albumid']
+            albumid = file.metadata.getall('musicbrainz_albumid')
+            albumid = albumid[0] if albumid else ''
             is_valid_albumid = mbid_validate(albumid)
 
             if is_valid_albumid and is_valid_recordingid:
                 log.debug("%r has release (%s) and recording (%s) MBIDs, moving to track...",
                           file, albumid, recordingid)
                 self.move_file_to_track(file, albumid, recordingid)
-                return
-
-            if is_valid_albumid:
+                file_moved = True
+            elif is_valid_albumid:
                 log.debug("%r has only release MBID (%s), moving to album...",
                           file, albumid)
                 self.move_file_to_album(file, albumid)
-                return
-
-            if is_valid_recordingid:
+                file_moved = True
+            elif is_valid_recordingid:
                 log.debug("%r has only recording MBID (%s), moving to non-album track...",
                           file, recordingid)
                 self.move_file_to_nat(file, recordingid)
-                return
+                file_moved = True
 
-        if target:
-            self.move_file(file, target)
-        else:
-            self.unclustered_files.add_file(file)
+        if not file_moved:
+            target = self.move_file(file, target)
+            if target and target != self.unclustered_files:
+                file_moved = True
+
+        if not file_moved and unmatched_files is not None:
+            unmatched_files.append(file)
 
         # fallback on analyze if nothing else worked
-        if config.setting['analyze_new_files'] and file.can_analyze():
+        if not file_moved and config.setting['analyze_new_files'] and file.can_analyze():
             log.debug("Trying to analyze %r ...", file)
             self.analyze([file])
 
+        # Auto cluster newly added files if they are not explicitly moved elsewhere
+        if self._pending_files_count == 0 and unmatched_files and config.setting["cluster_new_files"]:
+            self.cluster(unmatched_files)
+
     def move_file(self, file, target):
-        if target is None:
-            log.debug("Aborting move since target is invalid")
-            return
+        """Moves a file to target, if possible
+
+        Returns the actual target the files has been moved to or None
+        """
         if isinstance(target, Album):
             self.move_files_to_album([file], album=target)
-        elif isinstance(target, ClusterList):
-            if isinstance(file.parent, Track):
-                file.parent.remove_file(file)
-            self.cluster([file])
-        # To be able to move a file to it target must implement add_file(file)
-        elif hasattr(target, 'add_file'):
-            file.move(target)
+        else:
+            if isinstance(target, File) and target.parent:
+                target = target.parent
+            if not file.move(target):
+                # Ensure a file always has a parent so it shows up in UI
+                if not file.parent:
+                    target = self.unclustered_files
+                    file.move(target)
+                # Unsupported target, do not move the file
+                else:
+                    target = None
+        return target
 
     def move_files(self, files, target, move_to_multi_tracks=True):
         if target is None:
@@ -497,15 +525,13 @@ class Tagger(QtWidgets.QApplication):
         elif isinstance(target, Album):
             self.move_files_to_album(files, album=target)
         elif isinstance(target, ClusterList):
-            for file in process_events_iter(files):
-                if isinstance(file.parent, Track):
-                    file.parent.remove_file(file)
             self.cluster(files)
         self.window.set_sorting(True)
 
     def add_files(self, filenames, target=None):
         """Add files to the tagger."""
         ignoreregex = None
+        config = get_config()
         pattern = config.setting['ignore_regex']
         if pattern:
             try:
@@ -515,7 +541,7 @@ class Tagger(QtWidgets.QApplication):
         ignore_hidden = config.setting["ignore_hidden_files"]
         new_files = []
         for filename in filenames:
-            filename = os.path.normpath(os.path.realpath(filename))
+            filename = normpath(filename)
             if ignore_hidden and is_hidden(filename):
                 log.debug("File ignored (hidden): %r" % (filename))
                 continue
@@ -537,8 +563,9 @@ class Tagger(QtWidgets.QApplication):
             new_files.sort(key=lambda x: x.filename)
             self.window.set_sorting(False)
             self._pending_files_count += len(new_files)
+            unmatched_files = []
             for i, file in enumerate(new_files):
-                file.load(partial(self._file_loaded, target=target))
+                file.load(partial(self._file_loaded, target=target, unmatched_files=unmatched_files))
                 # Calling processEvents helps processing the _file_loaded
                 # callbacks in between, which keeps the UI more responsive.
                 # Avoid calling it to often to not slow down the loading to much
@@ -567,6 +594,7 @@ class Tagger(QtWidgets.QApplication):
                 log.warning(err)
 
     def add_paths(self, paths, target=None):
+        config = get_config()
         files = self._scan_paths_recursive(paths,
                             config.setting['recursively_add_files'],
                             config.setting["ignore_hidden_files"])
@@ -574,13 +602,14 @@ class Tagger(QtWidgets.QApplication):
 
     def get_file_lookup(self):
         """Return a FileLookup object."""
+        config = get_config()
         return FileLookup(self, config.setting["server_host"],
                           config.setting["server_port"],
                           self.browser_integration.port)
 
     def copy_files(self, objects):
         mimeData = QtCore.QMimeData()
-        mimeData.setUrls([QtCore.QUrl.fromLocalFile(f.filename) for f in (self.get_files_from_objects(objects))])
+        mimeData.setUrls([QtCore.QUrl.fromLocalFile(f.filename) for f in iter_files_from_objects(objects)])
         self.clipboard().setMimeData(mimeData)
 
     def paste_files(self, target):
@@ -608,6 +637,7 @@ class Tagger(QtWidgets.QApplication):
             return
         search = search_types[search_type]
         lookup = self.get_file_lookup()
+        config = get_config()
         if config.setting["builtin_search"] and not force_browser:
             if not lookup.mbid_lookup(text, search['entity'],
                                       mbid_matched_callback=mbid_matched_callback):
@@ -620,6 +650,7 @@ class Tagger(QtWidgets.QApplication):
     def collection_lookup(self):
         """Lookup the users collections on the MusicBrainz website."""
         lookup = self.get_file_lookup()
+        config = get_config()
         lookup.collection_lookup(config.persist["oauth_username"])
 
     def browser_lookup(self, item):
@@ -643,15 +674,26 @@ class Tagger(QtWidgets.QApplication):
                 item.filename if isinstance(item, File) else '')
 
     def get_files_from_objects(self, objects, save=False):
-        """Return list of files from list of albums, clusters, tracks or files."""
-        return uniqify(chain(*[obj.iterfiles(save) for obj in objects]))
+        """Return list of unique files from list of albums, clusters, tracks or files.
+
+        Note: Consider using picard.util.iter_files_from_objects instead, which returns an iterator.
+        """
+        return list(iter_files_from_objects(objects, save=save))
 
     def save(self, objects):
         """Save the specified objects."""
-        files = self.get_files_from_objects(objects, save=True)
-        for file in files:
+        for file in iter_files_from_objects(objects, save=True):
             file.save()
 
+    def load_mbid(self, type, mbid):
+        self.bring_tagger_front()
+        if type == 'album':
+            self.load_album(mbid)
+        elif type == 'nat':
+            self.load_nat(mbid)
+        else:
+            log.warning('Unknown type to load: %s', type)
+
     def load_album(self, album_id, discid=None):
         album_id = self.mbid_redirects.get(album_id, album_id)
         album = self.albums.get(album_id)
@@ -705,7 +747,7 @@ class Tagger(QtWidgets.QApplication):
         if album.id not in self.albums:
             return
         album.stop_loading()
-        self.remove_files(self.get_files_from_objects([album]))
+        self.remove_files(list(album.iterfiles()))
         del self.albums[album.id]
         if album.release_group:
             album.release_group.remove_album(album.id)
@@ -718,7 +760,9 @@ class Tagger(QtWidgets.QApplication):
     def remove_nat(self, track):
         """Remove the specified non-album track."""
         log.debug("Removing %r", track)
-        self.remove_files(self.get_files_from_objects([track]))
+        self.remove_files(list(track.iterfiles()))
+        if not self.nats:
+            return
         self.nats.tracks.remove(track)
         if not self.nats.tracks:
             self.remove_album(self.nats)
@@ -739,29 +783,30 @@ class Tagger(QtWidgets.QApplication):
     def remove(self, objects):
         """Remove the specified objects."""
         files = []
-        for obj in objects:
-            if isinstance(obj, File):
-                files.append(obj)
-            elif isinstance(obj, NonAlbumTrack):
-                self.remove_nat(obj)
-            elif isinstance(obj, Track):
-                files.extend(obj.linked_files)
-            elif isinstance(obj, Album):
-                self.window.set_statusbar_message(
-                    N_("Removing album %(id)s: %(artist)s - %(album)s"),
-                    {
-                        'id': obj.id,
-                        'artist': obj.metadata['albumartist'],
-                        'album': obj.metadata['album']
-                    }
-                )
-                self.remove_album(obj)
-            elif isinstance(obj, UnclusteredFiles):
-                files.extend(list(obj.files))
-            elif isinstance(obj, Cluster):
-                self.remove_cluster(obj)
-        if files:
-            self.remove_files(files)
+        with self.window.ignore_selection_changes:
+            for obj in objects:
+                if isinstance(obj, File):
+                    files.append(obj)
+                elif isinstance(obj, NonAlbumTrack):
+                    self.remove_nat(obj)
+                elif isinstance(obj, Track):
+                    files.extend(obj.files)
+                elif isinstance(obj, Album):
+                    self.window.set_statusbar_message(
+                        N_("Removing album %(id)s: %(artist)s - %(album)s"),
+                        {
+                            'id': obj.id,
+                            'artist': obj.metadata['albumartist'],
+                            'album': obj.metadata['album']
+                        }
+                    )
+                    self.remove_album(obj)
+                elif isinstance(obj, UnclusteredFiles):
+                    files.extend(list(obj.files))
+                elif isinstance(obj, Cluster):
+                    self.remove_cluster(obj)
+            if files:
+                self.remove_files(files)
 
     def _lookup_disc(self, disc, result=None, error=None):
         self.restore_cursor()
@@ -773,6 +818,7 @@ class Tagger(QtWidgets.QApplication):
 
     def lookup_cd(self, action):
         """Reads CD from the selected drive and tries to lookup the DiscID on MusicBrainz."""
+        config = get_config()
         if isinstance(action, QtWidgets.QAction):
             device = action.data()
         elif config.setting["cd_lookup_device"] != '':
@@ -790,31 +836,61 @@ class Tagger(QtWidgets.QApplication):
 
     @property
     def use_acoustid(self):
+        config = get_config()
         return config.setting["fingerprinting_system"] == "acoustid"
 
     def analyze(self, objs):
         """Analyze the file(s)."""
         if not self.use_acoustid:
             return
-        files = self.get_files_from_objects(objs)
-        for file in files:
-            file.set_pending()
-            self._acoustid.analyze(file, partial(file._lookup_finished,
-                                                 File.LOOKUP_ACOUSTID))
+        for file in iter_files_from_objects(objs):
+            if file.can_analyze():
+                file.set_pending()
+                self._acoustid.analyze(file, partial(file._lookup_finished, File.LOOKUP_ACOUSTID))
 
     def generate_fingerprints(self, objs):
         """Generate the fingerprints without matching the files."""
         if not self.use_acoustid:
             return
-        files = self.get_files_from_objects(objs)
 
         def finished(file, result):
             file.clear_pending()
 
-        for file in files:
+        for file in iter_files_from_objects(objs):
             file.set_pending()
             self._acoustid.fingerprint(file, partial(finished, file))
 
+    def extract_and_submit_acousticbrainz_features(self, objs):
+        """Extract AcousticBrainz features and submit them."""
+        if not self.ab_extractor.available():
+            return
+
+        for file in iter_files_from_objects(objs):
+            # Skip unmatched files
+            if not file.can_extract():
+                log.warning("AcousticBrainz requires a MusicBrainz Recording ID, but file does not have it: %s" % file.filename)
+            # And process matched ones
+            else:
+                file.set_pending()
+
+                # Check if file was either already processed or sent to the AcousticBrainz server
+                if file.acousticbrainz_features_file:
+                    results = (file.acousticbrainz_features_file, 0, "Writing results")
+                    ab_extractor_callback(self, file, results, False)
+                elif file.acousticbrainz_is_duplicate:
+                    results = (None, 0, "Duplicate")
+                    ab_extractor_callback(self, file, results, False)
+                else:
+                    file.acousticbrainz_error = False
+                    # Launch the acousticbrainz on a separate process
+                    log.debug("Extracting AcousticBrainz features from %s" % file.filename)
+                    ab_feature_extraction(
+                        self,
+                        file.metadata["musicbrainz_recordingid"],
+                        file.filename,
+                        partial(ab_extractor_callback, self, file)
+                    )
+
     # =======================================================================
     #  Metadata-based lookups
     # =======================================================================
@@ -828,22 +904,43 @@ class Tagger(QtWidgets.QApplication):
     #  Clusters
     # =======================================================================
 
-    def cluster(self, objs):
+    def cluster(self, objs, callback=None):
         """Group files with similar metadata to 'clusters'."""
         log.debug("Clustering %r", objs)
-        if len(objs) <= 1 or self.unclustered_files in objs:
-            files = list(self.unclustered_files.files)
+        files = iter_files_from_objects(objs)
+        try:
+            file = next(files)
+        except StopIteration:
+            files = self.unclustered_files.files
         else:
-            files = self.get_files_from_objects(objs)
+            files = itertools.chain([file], files)
+        thread.run_task(
+            partial(self._do_clustering, list(files)),
+            partial(self._clustering_finished, callback))
 
-        self.window.set_sorting(False)
-        cluster_files = defaultdict(list)
-        for name, artist, files in Cluster.cluster(files, 1.0, self):
-            cluster = self.load_cluster(name, artist)
-            cluster_files[cluster].extend(files)
-        for cluster, files in process_events_iter(cluster_files.items()):
-            cluster.add_files(files)
-        self.window.set_sorting(True)
+    def _do_clustering(self, files):
+        # The clustering algorithm should completely run in the thread,
+        # hence do not return the iterator.
+        return list(Cluster.cluster(files))
+
+    def _clustering_finished(self, callback, result=None, error=None):
+        if error:
+            log.error('Error while clustering: %r', error)
+            return
+
+        with self.window.ignore_selection_changes:
+            self.window.set_sorting(False)
+            for file_cluster in process_events_iter(result):
+                files = set(file_cluster.files)
+                if len(files) > 1:
+                    cluster = self.load_cluster(file_cluster.title, file_cluster.artist)
+                else:
+                    cluster = self.unclustered_files
+                cluster.add_files(files)
+            self.window.set_sorting(True)
+
+        if callback:
+            callback()
 
     def load_cluster(self, name, artist):
         for cluster in self.clusters:
@@ -926,6 +1023,8 @@ def process_picard_args():
                         help="do not restore positions and/or sizes")
     parser.add_argument("-P", "--no-plugins", action='store_true',
                         help="do not load any plugins")
+    parser.add_argument("--no-crash-dialog", action='store_true',
+                        help="disable the crash dialog")
     parser.add_argument('-v', '--version', action='store_true',
                         help="display version information and exit")
     parser.add_argument("-V", "--long-version", action='store_true',
@@ -945,8 +1044,9 @@ def main(localedir=None, autoupdate=True):
     QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps)
     QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling)
     # HighDpiScaleFactorRoundingPolicy is available since Qt 5.14. This is
-    # required to support fractional scaling properly.
-    if hasattr(QtGui.QGuiApplication, 'setHighDpiScaleFactorRoundingPolicy'):
+    # required to support fractional scaling on Windows properly.
+    # It causes issues without scaling on Linux, see https://tickets.metabrainz.org/browse/PICARD-1948
+    if IS_WIN and hasattr(QtGui.QGuiApplication, 'setHighDpiScaleFactorRoundingPolicy'):
         QtGui.QGuiApplication.setHighDpiScaleFactorRoundingPolicy(
             QtCore.Qt.HighDpiScaleFactorRoundingPolicy.PassThrough)
 
